/*
 * main.c:  main C entry
 *   by uratan! 2008.9.8
 */
#include "common.h"
#include "LPC214x.h"
#include "irq.h"

/*
 * switch uart0/uart1 here, sync with the same macro in uart1.c
 */
#define USE_UART1   1


/*
 * functions I have
 */
int main(void);
void sys_init(void);
void IRQ_handler(void)   __attribute__ ((interrupt("IRQ")));
void FIQ_handler(void)   __attribute__ ((interrupt("FIQ")));
//void SWI_handler(void)   __attribute__ ((interrupt("SWI")));
void UNDEF_handler(void) __attribute__ ((interrupt("UNDEF")));
void PABT_handler(void)  __attribute__ ((interrupt("ABORT")));
void DABT_handler(void)  __attribute__ ((interrupt("ABORT")));

/*
 * externs
 */
/* in timer0.c */
extern int timer0_init(void);
extern void timer0_run(void);
extern void timer0_stop(void);
extern volatile int timer0_tick;
/* in uart1.c */
extern int uart1_init(u_long baudrate);
extern short peek_recv(void);
extern void set_to_send(u_char c);

/*
 * variables
 */
int counter = 0;


/*
 * main entry
 */
int main(void)
{
	/*
	 * setup misc system configuration
	 */
	sys_init();
	init_VIC();

	/*
	 * initialize I/O pins
	 */
	SCS |= 0x03;		/* both GPIO are used as Fast I/O */
	PINSEL0 = 0;
	PINSEL1 = 0;
	FIO0DIR = 0x00000000;
	PINSEL2 = 0;
	FIO1DIR = 0x00000000;

	/*
	 * setup debug UART port
	 */
#if USE_UART1
	cm_init_serial0(57600);			/* setup debug uart port */
	rprintf_devopen(cm_putchar);	/* init rprintf() */
	rprintf("rprintf: %s", "Hello from rprintf()\r\n");
#endif /* USE_UART1 */
{
extern u_long *HIDDscr1, *ReportDscr1, *ReportDscr1Len;
rprintf("HIDDscr1:       0x%08x\r\n", &HIDDscr1);		/* 0x0214 */
rprintf("ReportDscr1:    0x%08x\r\n", &ReportDscr1);	/* 0x0256 */
rprintf("ReportDscr1Len: 0x%08x\r\n", &ReportDscr1Len);	/* 0x003d */
}

	/*
	 * to drive 'STAT' LED
	 */
	FIO1DIR |= (1<<24);

	/*
	 *
	 */
	uart1_init(9600);
	set_to_send('1');
	set_to_send('u');
	set_to_send('2');

	/*
	 * start timer
	 */
	timer0_init();
	timer0_run();

	/*
	 * The main Function is an endless loop
	 */
	while(1) {
		int k;

#if USE_UART1
		k = cm_keyin();
		if(k != -1) {
			cm_putchar(k);
		}
#endif /* USE_UART1 */

		if(timer0_tick) {
			timer0_tick = FALSE;

			if(++counter >= 500) {
	set_to_send('5');
#if !USE_UART1
	while(1) {
		k = peek_recv();
		if(k == -1)
			break;
		set_to_send(k);
	}
#endif /* !USE_UART1 */
				counter = 0;

				/*
				 * toggle LED every 500msec
				 */
				if(FIO1PIN & (1<<24)) {
					FIO1CLR = (1<<24);
				} else {
					FIO1SET = (1<<24);
				}
			}
		}
	}

	/* never return from main() */
	while(1)
		;
}


/* ===============================================================
 * setup core hardware running conditions
 */
void sys_init(void)
{
	/*
	 * select vector memory
	 */
	MEMMAP = 0x01;		/* User Flash Mode */

	/*
	 * Setting peripheral Clock (PCLK) to System Clock (CCLK)
	 *   0: PCLK = 1 * CCLK / 4
	 *   1: PCLK = 4 * CCLK / 4
	 *   2: PCLK = 2 * CCLK / 4
	 */
#if (Fpclk / (Fcclk / 4)) == 1
	VPBDIV = 0;						/* <== 15MHz */
#elif (Fpclk / (Fcclk / 4)) == 2
	VPBDIV = 2;
#elif (Fpclk / (Fcclk / 4)) == 4
	VPBDIV = 1;
#else
#error invalid VPBDIV
#endif

	/*
	 * initial PLL for system clock
	 */
	PLLCON = 1;

#if (Fcco / Fcclk) == 2
	PLLCFG = ((Fcclk / Fosc) - 1) | (0 << 5);
#elif (Fcco / Fcclk) == 4
	PLLCFG = ((Fcclk / Fosc) - 1) | (1 << 5);			/* <== */
#elif (Fcco / Fcclk) == 8
	PLLCFG = ((Fcclk / Fosc) - 1) | (2 << 5);
#elif (Fcco / Fcclk) == 16
	PLLCFG = ((Fcclk / Fosc) - 1) | (3 << 5);
#else
#error invalid PLLCFG
#endif
	PLLFEED = 0xaa;
	PLLFEED = 0x55;

	while((PLLSTAT & (1 << 10)) == 0)
		;

	PLLCON = 3;
	PLLFEED = 0xaa;
	PLLFEED = 0x55;

	/*
	 * config memory accelerater module
	 */
	MAMCR = 0;

#if Fcclk < 20000000
	MAMTIM = 1;
#elif Fcclk < 40000000
	MAMTIM = 2;
#else
	MAMTIM = 3;				/* <== */
#endif

	MAMCR = 2;

	return;
}

/* ===============================================================
 * exception handlers
 */
void IRQ_handler(void)
{
if(USE_UART1)
	rprintf("!!IRQ\r\n");
	while(1)
		;		/* halt at here */
}

void FIQ_handler(void)
{
if(USE_UART1)
	rprintf("!!FIQ\r\n");
	while(1)
		;		/* halt at here */
}

void UNDEF_handler(void)
{
if(USE_UART1)
	rprintf("!!UNDEF\r\n");
	while(1)
		;		/* halt at here */
}

void PABT_handler(void)
{
if(USE_UART1)
	rprintf("!!PABT\r\n");
	while(1)
		;		/* halt at here */
}

void DABT_handler(void)
{
if(USE_UART1)
	rprintf("!!DABT\r\n");
	while(1)
		;		/* halt at here */
}
